#include "SoundClass.h"


SoundClass::SoundClass()
{
    musicBuffer=0;
    musicFadeBuffer=0;
    musicFadeAmp=0.0;
    sfxCursor=0;
    for(int i=0;i<16;i++)
    {
        sfxBuffers[i]=0;
    }
    musicBuffer=0;
    //ctor
}

SoundClass::~SoundClass()
{
    //dtor
}

SDL_AudioDeviceID SoundClass::dev;
void SoundClass::sdl_init()
{
    SDL_Init(SDL_INIT_EVERYTHING);

    SDL_AudioSpec want,have;
    SDL_zero(want);
    want.freq=44100;
    want.format=AUDIO_F32SYS;
    want.channels = 2;
    want.samples = 4096;
    want.callback = sdl_sound_callback;
    want.userdata=this;
    dev=SDL_OpenAudioDevice(0,0,&want,&have,0);

    SDL_PauseAudioDevice(dev,0);

}
void SoundClass::sdl_quit()
{
    SDL_PauseAudioDevice(dev,1);
    SDL_CloseAudioDevice(dev);
}
void SoundClass::sdl_sound_callback(void*user,unsigned char*output,int dataLng)
{
    ((SoundClass*)user)->callback((float*)output,dataLng/8);// float32 stereo
}


void SoundClass::playSfx(SoundBuffer* buffer)
{
    SDL_LockAudioDevice(dev);
    if(buffer)
    {
        sfxOffsets[sfxCursor]=0;
        sfxBuffers[sfxCursor]=buffer;


        this->sfxCursor++;
        this->sfxCursor%=16;
    }
    SDL_UnlockAudioDevice(dev);
}
void SoundClass::callback(float*out,int frames)
{
    for(int i=0;i<frames;i++)
    {
        out[(i<<1) + 0]=0;
        out[(i<<1) + 1]=0;
    }
    //sample
    for(int i=0;i<16;i++)
    {
        if(sfxBuffers[i])
        {
            sfxBuffers[i]->addToStereo(out,sfxOffsets[i],frames,0.3,NULL);
        }
    }

    // muzyka
   {
        if(musicBuffer)
            musicBuffer->addToStereo(out,musicBufferOffset,frames,1.0,NULL,true);

        if(musicFadeBuffer)
            musicFadeBuffer->addToStereo(out,musicFadeBufferOffset,frames, fmax(musicFadeAmp*0.9,0.0), &musicFadeAmp, true);
    }

    /*if(gv)
    {
        for(int i=0;i<frames;i++)
        {
            gbufferL[gbw>>8]=out[(i<<1) + 0];
            gbufferR[gbw>>8]=out[(i<<1) + 1];
            gbr+=256+gv;
            gbw+=256;
            out[(i<<1) + 0]=gbufferL[gbr>>8];
            out[(i<<1) + 1]=gbufferR[gbr>>8];
        }
    }*/


    // rozpizdziel
    /*for(int i=0;i<frames;i++)
    {
        out[(i<<1) + 0]=out[(i<<1) + 0] + out[(i<<1) + 0]>0.0?-1.0:1.0;
        out[(i<<1) + 1]=out[(i<<1) + 0] + out[(i<<1) + 1]>0.0?-1.0:1.0;
    }*/
}
void SoundClass::selectMusic(SoundBuffer * music)
{
    SDL_LockAudioDevice(dev);
    musicBuffer=music;
    musicBufferOffset=0;
    SDL_UnlockAudioDevice(dev);
}
void SoundClass::stopMusic()
{
    SDL_LockAudioDevice(dev);
    musicFadeBuffer=musicBuffer;
    musicBuffer = 0;
    musicFadeBufferOffset = musicBufferOffset;
    musicFadeAmp=1.0;
    SDL_UnlockAudioDevice(dev);
}

void SoundBuffer::addToStereo(float * out, unsigned int &offset,unsigned int buffLng,float amp, float *iamp,bool loop)
{
    float i2amp;
    if(iamp)
    {
        i2amp=*iamp;
    }
    else
    {
        i2amp=amp;
    }
    float ddamp=(amp-i2amp)/(float)buffLng;
    if(this->stereo)
    {
        for(unsigned int i=0;i<buffLng;i++)
        {
            if(!loop && offset>=this->frames)
                break;
            if( loop && offset>=this->frames)
                offset=0;
            i2amp+=ddamp;
            *(out++)+=this->data[(offset<<1) + 0]* (i2amp);
            *(out++)+=this->data[(offset<<1) + 1]* (i2amp);
            offset++;
        }
    }
    else
    {
        for(unsigned int i=0;i<buffLng;i++)
        {
            if(!loop && offset>=this->frames)
                break;
            if( loop && offset>=this->frames)
                offset=0;
            i2amp+=ddamp;
            *(out++)+=this->data[offset]* (i2amp);
            *(out++)+=this->data[offset]* (i2amp);
            offset++;
        }
    }
    if(iamp)
    {
        *iamp=i2amp;
    }
}
