#ifndef SOUNDCLASS_H
#define SOUNDCLASS_H
#include <SDL2/SDL.h>
#include <stdio.h>
#include <math.h>

class SoundBuffer;
class SoundClass
{
    unsigned int musicBufferOffset;
    unsigned int musicFadeBufferOffset;
    float musicFadeAmp;
    SoundBuffer * musicBuffer;
    SoundBuffer * musicFadeBuffer;



    unsigned int sfxCursor;
    SoundBuffer *sfxBuffers[16];
    unsigned int sfxOffsets[16];

public:
    SoundClass();
    ~SoundClass();
    static SDL_AudioDeviceID dev;
    void sdl_init();
    void sdl_quit();
    static inline void sdl_sound_callback(void*user,unsigned char*output,int dataLng);
    void callback(float*out,int frames);
    void selectMusic(SoundBuffer*);
    void stopMusic();


    void playSfx(SoundBuffer*);
    float gbufferL[256];
    float gbufferR[256];
    unsigned short gbr;
    unsigned short gbw;
    unsigned short gv;
    bool gon;
};


class SoundBuffer
{
public:
    bool stereo;
    inline SoundBuffer(const char * path = 0 ,bool stereo=false)
    {
        data=0;
        if(path)
            load(path,stereo);
    }
    inline void load(const char * path,bool stereo)
    {
        unload();

        FILE *fp = fopen(path, "rb");
        fseek(fp, 0, SEEK_END);
        unsigned int lng = ftell(fp);
        fseek(fp, 0, SEEK_SET);

        this->data = new float[lng/4];
        fread(data, lng, 1, fp);
        fclose(fp);
        if(stereo)
            this->frames = lng/8;
        else
            this->frames = lng/4;
        this->stereo = stereo;
    }
    inline void unload()
    {
        if(data)
            delete[]data;
        data=0;
    }
    float * data;
    unsigned int frames;

    void addToStereo(float * out, unsigned int &offset,unsigned int buffLng,float amp, float *iamp=NULL,bool loop=false);
};

#endif // SOUNDCLASS_H
